<!-- FuseMinder.cfm by jeff.peters@bigfoot.com -->

<!---
|| FUSEDOC ||
	
|| General ||
Name: FuseMinder.cfm (formerly MindReader)
Version: 1.1
Author: jeff.peters@bigfoot.com

|| History ||
20 Oct 2000 - Release 1.0
12 Dec 2000 - Release 1.1

|| Responsibilities ||
I read a text outline file of the form generated by Visual Mind and generate the 
specified circuit apps, including fuseboxes and fusestubs.  
If "attributes.location" is not defined, I use the current directory as home.  
If "attributes.verbose" is TRUE or not defined, I produce progress statements 
while working.  If "attributes.indenter" is not defined, I use two spaces ("  ") 
as the outline indenting string.

|| Attributes ||
--> attributes.outlineFile: a STRING ( name of outline file )+
--> attributes.location: a STRING ( location of outline file )
--> attributes.verbose: a BOOLEAN ( FALSE ) 
--> attributes.indenter: a STRING ( string to use for indenting outline levels )

|| END FUSEDOC ||--->
<cfparam name="attributes.OutlineFile" default="DACF.txt">
<cfparam name="attributes.verbose" default="True">
<cfparam name="attributes.location" default="#GetDirectoryFromPath(GetCurrentTemplatePath())#">
<cfparam name="attributes.indenter" default="  ">

<cfif attributes.verbose EQ "Yes">
FuseMinder v 1.1 by jeff.peters@bigfoot.com - rundate:<cfoutput>#Dateformat(Now())#, #TimeFormat(Now())#</cfoutput>
<hr>
</cfif>

<cfif IsDefined("attributes.OutlineFile") EQ "No">
  <cfthrow type="MindReaderError" message="OutlineFile not specified.">
</cfif>

<cfset homeRoot = "#attributes.location#">

<cfset Crlf = "#Chr(10)#">
<!--- If homeRoot has a colon in it, we're in Windows, else Unix --->
<cfif Find(":",homeRoot) GT 0>
  <cfset slash = "\">
<cfelse>
  <cfset slash = "/">
</cfif>

<cfset lastType = "">
<cfset lastCircuit = "">
<cfset lastFuseaction = "">
<cfset fbOpenLines = "<cfswitch expression='fuseaction'>#Crlf#">
<cfset modeEverybodyReadWrite = 666>

<!--- Indenting string constants --->
<cfset indenterCircuit = RepeatString(#attributes.indenter#,1)>
<cfset indenterFuseaction = RepeatString(#attributes.indenter#,2)>
<cfset indenterFuse = RepeatString(#attributes.indenter#,3)>
<cfset indenterFusedoc = RepeatString(#attributes.indenter#,4)>

<!--- Home fusebox string constants --->
<cfset hfbOpenLines = "<cfswitch expression='##ListFirst(fuseaction,'.')##'>">
<cfset hfbCloseLines = "  <cfdefaultcase>#Crlf#    No handler for this fuseaction.#Crlf#  </cfdefaultcase>#Crlf#</cfswitch>">

<!--- Fusebox file string constants --->
<cfset fbOpenLines = "<cfswitch expression='##ListLast(fuseaction,'.')##'>">
<cfset fbCloseLines = "  </cfcase>#Crlf#  <cfdefaultcase>#Crlf#    No handler for this fuseaction.#Crlf#  </cfdefaultcase>#Crlf#</cfswitch>">
<cfset fbOpenCaseStart = "  <cfcase value='">
<cfset fbOpenCaseEnd = "'>">
<cfset fbIncludeStart = "    <cfinclude template='">
<cfset fbIncludeEnd = "'>">
<cfset fbCloseCase = "  </cfcase>">

<!--- Fuse file string constants --->
<cfset ffNoteLineStart = "<!-- ">
<cfset ffNoteLineEnd = " -->">
<cfset ffAuthor = "">

<cfset aryCircuitNames = ArrayNew(1)>

<!--- Read in text file, ignoring header info --->
<cffile action="read" variable="strRawInput" file="#homeRoot##attributes.OutlineFile#">
<cfset aryOutline = ListToArray(strRawInput,Crlf)>

<!--- For each line in spec: --->
<cfloop from="1" to="#ArrayLen(aryOutline)#" index="lineNum">
	<!--- Determine line type based on number of leading spaces --->
	<cfif Left(aryOutline[lineNum],8) EQ "#indenterFusedoc#">
	  <cfset lineType = "fusedoc">
	<cfelseif Left(aryOutline[lineNum],6) EQ "#indenterFuse#">
	  <cfset lineType = "fuse">
	<cfelseif Left(aryOutline[lineNum],4) EQ "#indenterFuseaction#">
	  <cfset lineType = "fuseaction">
	<cfelseif Left(aryOutline[lineNum],2) EQ "#indenterCircuit#">
	  <cfset lineType = "circuit">
	<cfelseif Left(aryOutline[lineNum],2) GT "#indenterCircuit#">
	  <cfset lineType = "homeapp">
	<cfelse>
	  <cfset lineType = "unknown">
	</cfif>

	<CFSWITCH expression="#lineType#">
	    <cfcase value="homeapp">
          <!--- Store name of home app --->
		  <cfset appLine = Trim(aryOutline[lineNum])>
		  <cfset appName = Trim(ListFirst(appLine,":"))>
		  <cfif FindNoCase(":",appLine) GT 0>
		    <cfset ffAuthor = " by #ListLast(appLine,":")#">
		  </cfif>
		  <cfif attributes.verbose EQ "Yes">
		    <cfoutput>Building files for #appName##ffAuthor#.<br></cfoutput>
		  </cfif>
		</cfcase>

		<cfcase value="circuit">
		  <cfset circuitIndex = ArrayLen(aryCircuitNames) + 1>
		  <!--- aryCircuitNames will be used to build the home fusebox at the end. --->
          <cfset circuitName = "#Trim(ListLast(aryOutline[lineNum],indenterCircuit))#">
		  <cfset aryCircuitNames[circuitIndex] = circuitName>
		  <cfset circuitPath="#homeRoot##circuitName#">

		  <!--- If lastCircuit is not this circuit name and not empty, write 
			    closing lines to last circuit's fusebox.--->
		  <cfif (lastCircuit GT "") AND (lastCircuit NEQ "#circuitName#")>

		    <cffile action="APPEND" 
			          file="#homeRoot##lastCircuit##slash#index.cfm"
			        output="#fbCloseLines#">
			<cfset circuitClosed = "Yes">
			<cfset lastCircuit = "#circuitName#">
		  </cfif>
          <!--- Create the circuit directory; ignore error if it already exists. --->
		  <cfset circuitClosed = "No">
		  <cftry>
            <cfdirectory action="create" 
			          directory="#circuitPath#">
			<cfcatch type="Any">
			</cfcatch>
		  </cftry>
		  <!--- Create index.cfm for circuit and write opening lines --->
		  <cftry>
		    <cffile action="read" file="#circuitPath##slash#index.cfm" variable="foo">
			<cfcatch type="Any">
			  <!--- Write note line (first line of file) to fusebox file --->
			  <cffile action="WRITE"
			            mode="#modeEverybodyReadWrite#"
	                    file="#circuitPath##slash#index.cfm"
				      output="#ffNoteLineStart#index.cfm for #circuitName# circuit#ffAuthor##ffNoteLineEnd#">
			  <cffile action="APPEND"
			                file="#circuitPath##slash#index.cfm"
		              output="#fbOpenLines#">
			</cfcatch>
	      </cftry>
		<cfif attributes.verbose EQ "Yes">
		  <cfoutput>Building #circuitName# circuit.<br></cfoutput>
		</cfif>
		</cfcase>

		<cfcase value="fuseaction">
		  <cfset fuseactionName = "#Trim(ListLast(aryOutline[lineNum],indenterFuseaction))#">
		  <!--- If lastFuseaction is not this fuseaction name and not empty, write 
			    case closing lines to appropriate circuit's fusebox.--->
		  <cfif ((lastFuseaction NEQ "") AND (lastFuseaction NEQ fuseactionName))>
			  <cfif ((lastCircuit NEQ "") AND (lastCircuit NEQ circuitName))>
			    <cffile action="APPEND" 
				          file="#homeRoot##lastCircuit##slash#index.cfm"
				        output="#fbCloseCase#">
			  <cfelse>
			    <cfif lastType NEQ "circuit">
				    <cffile action="APPEND" 
					          file="#circuitPath##slash#index.cfm"
					        output="#fbCloseCase#">
				</cfif>
			  </cfif>
			  <cfset lastCircuit = "#circuitName#">  
		  </cfif>
		  <!--- Append fuseaction's cfcase line to current circuit's fusebox --->
		  <cffile action="APPEND"
                    file="#circuitPath##slash#index.cfm"
			      output="#fbOpenCaseStart##fuseactionName##fbOpenCaseEnd#">
		  <!--- Set lastFuseaction = fusecactionName --->
		  <cfset lastFuseaction = "#fuseactionName#">
		  <cfif attributes.verbose EQ "Yes">
		    <cfoutput>&nbsp;&nbsp;Fuseaction #fuseactionName# written.<br></cfoutput>
		  </cfif>
		</cfcase>

		<cfcase value="fuse">
		  <cfset fuseFile = "#Trim(ListLast(aryOutline[lineNum],indenterFuse))#">
		  <!--- Append cfinclude line to fusebox for lastCircuit --->
		  <cffile action="APPEND"
                    file="#circuitPath##slash#index.cfm"
			      output="#fbIncludeStart##fuseFile##fbIncludeEnd#">
		  <!--- Write note line (first line of file) to fuse file --->
		  <cffile action="WRITE"
		            mode="#modeEverybodyReadWrite#"
                    file="#circuitPath##slash##fuseFile#"
			      output="#ffNoteLineStart##fuseFile##ffAuthor##ffNoteLineEnd#">
	      <cfif attributes.verbose EQ "Yes">
            <cfoutput>&nbsp;&nbsp;&nbsp;&nbsp;Fuse #fuseFile# written.<br></cfoutput>
		  </cfif>
		</cfcase>

		<cfcase value="fusedoc">
		  <!--- Append fusedoc line to fuseFile --->
		  <cffile action="APPEND"
                    file="#circuitPath##slash##fuseFile#"
			      output="#Replace(aryOutline[lineNum], indenterFusedoc, "")#">
		</cfcase>
		<cfdefaultcase>
		</cfdefaultcase>
	</cfswitch>
	<cfset lastType=lineType>
</cfloop>

<!-- If the last circuit wasn't closed, write closing lines to its file. -->
<cfif circuitClosed EQ "No">
  <cffile action="APPEND" 
	        file="#circuitPath##slash#index.cfm"
	      output="#fbCloseLines#">
</cfif>

<!-- Now write the home app's fusebox file --->
<!--- Write opening lines --->
<cftry>
  <cffile action="read" file="#homeRoot#index.cfm" variable="foo">
<cfcatch type="Any">
  <!--- Write note line (first line of file) to fusebox file --->
  <cffile action="WRITE"
            mode="#modeEverybodyReadWrite#"
            file="#homeRoot#index.cfm"
	      output="#ffNoteLineStart#index.cfm for #appName# application#ffAuthor##ffNoteLineEnd#">
  <cffile action="APPEND"
            file="#homeRoot#index.cfm"
          output="#hfbOpenLines#">
</cfcatch>
</cftry>
<!--- Write each case --->
<cfloop from="1" to="#ArrayLen(aryCircuitNames)#" index="i">
  <!--- Append circuit's cfcase lines to current circuit's fusebox --->
  <cffile action="APPEND"
            file="#homeRoot#index.cfm"
	      output="#fbOpenCaseStart##Left(aryCircuitNames[i],4)##fbOpenCaseEnd#">

  <cffile action="APPEND"
            file="#homeRoot#index.cfm"  output="#fbIncludeStart##aryCircuitNames[i]##slash#index.cfm?fuseaction=##fuseaction###fbIncludeEnd#">
		  
  <cffile action="APPEND" 
            file="#homeRoot#index.cfm"
          output="#fbCloseCase#">
</cfloop>
<!--- Write closing lines --->
<cffile action="APPEND" 
          file="#homeRoot#index.cfm"
        output="#hfbCloseLines#">

<cfif attributes.verbose EQ "Yes">
  <cfoutput>Fusebox file for #appName# application written.</cfoutput>
</cfif>
		
<!-- Output end of process message --->
<cfif attributes.verbose EQ "Yes">
  <cfoutput>Application build complete.<br></cfoutput>
  <hr>
  &copy; 2000 All Rights Reserved
</cfif>
